# Copyright (C) 2012, Clément Pit--Claudel (http://pit-claudel.fr/clement/blog)
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of 
# this software and associated documentation files (the "Software"), to deal in 
# the Software without restriction, including without limitation the rights to 
# use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
# the Software, and to permit persons to whom the Software is furnished to do so, 
# subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all 
# copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


# <h2>Shuffling text</h2>

import re, random
 
def shuffle_one(word):
  if len(word) <= 3:
    return word
  
  middle = list(word[1:-1])
  random.shuffle(middle)
   
  return word[0] + ''.join(middle) + word[-1]
 
def repl(match):
  return shuffle_one(match.groups()[0])
 
def shuffle(text):
  return re.sub("(\w+)", repl, text)
 
shuffle("Normally, reading this shouldn't be too hard.")

# <h2>Unshuffling scrambled text</h2>

def signature(word):
  word = word.lower()

  if len(word) <= 3:
    return word
   
  middle = list(word[1:-1])
  return word[0] + ''.join(sorted(middle)) + word[-1]
 
signature("Welcome") # 'wcelmoe'
signature("Woclmee") # 'wcelmoe'


handler = open("english.txt", encoding = "utf-8")
words = [word.strip() for word in handler.readlines()]
handler.close()
 
# Load dictionary entries
dictionary = {signature(word):word for word in words}


def unshuffle_one(word):
  sig = signature(word)
  return dictionary[sig] if sig in dictionary else word
   
def unrepl(match):
  return unshuffle_one(match.groups()[0])
 
def unshuffle(text):
  return re.sub("(\w+)", unrepl, text)
 
print(unshuffle("The eofrft rreuqied to sfssllcceuuy dhiepecr sbecmrald "
                "pgsaases daiaarclltmy ianserecs as wdors get lehegtinr."))

# <h2>Further thoughts</h2>
# <h3>Handling signature collisions</h3>

# Load dictionary entries
dictionary = {}
for word in words:
  sig = signature(word)

  if not sig in dictionary:
    dictionary[sig] = [] # Create an empty list
  
  # Add word to the list of all words sharing signature sig
  dictionary[sig].append(word)

def unshuffle_one(word):
  sig = signature(word)
  return '|'.join(dictionary[sig]) if sig in dictionary else word


def find_collisions():
  for sig in dictionary:
    if len(dictionary[sig]) > 1:
      print(', '.join(dictionary[sig]))

